import {
  Inject,
  Injectable,
  ConflictException,
  NotFoundException,
  ForbiddenException,
} from '@nestjs/common'
import { securityConfig, SecurityConfig } from 'src/common/configs'
import { MailerService } from '@nestjs-modules/mailer'
import { JwtService } from '@nestjs/jwt'
import { PrismaService } from 'nestjs-prisma'
import { EmailScene, EmailVerifycationDto, EmailSendDto } from './dto/email.dto'
import { UserEntity } from 'src/users/entities/user.entity'

@Injectable()
export class EmailService {
  private subjectMap = {
    [EmailScene.register]: '注册账号',
    [EmailScene.forgetPassword]: '找回密码',
    [EmailScene.changeEmail]: '修改邮箱',
    [EmailScene.deleteUser]: '删除用户',
  }
  constructor(
    private prismaService: PrismaService,
    private mailerService: MailerService,
    private jwtService: JwtService,
    @Inject(securityConfig.KEY)
    private secureConfig: SecurityConfig,
  ) {}

  async sendEmailToken(email: string, scene: EmailScene) {
    const user: UserEntity | null = await this.prismaService.user.findUnique({
      where: { email },
    })
    switch (scene) {
      case EmailScene.register:
      case EmailScene.changeEmail:
        if (user) {
          throw new ConflictException(`邮箱${email}已注册`)
        }
        break
      case EmailScene.forgetPassword:
      case EmailScene.deleteUser:
        if (!user) {
          throw new NotFoundException(`用户${email}不存在`)
        }
        break
    }

    const verifyCode = this.generateVerifyCode()
    const token = this.jwtService.sign(
      { email, scene },
      { secret: this.getEmailJwtSecret(verifyCode, scene) },
    )
    await this.mailerService.sendMail({
      to: email,
      subject: `【qiuxu.site】${this.subjectMap[scene]}`,
      html: `您正在qiuxu.site${this.subjectMap[scene]}，验证码为 <strong>${verifyCode}</strong>，30分钟内有效`,
    })
    return { token, userId: user?.id }
  }

  async verifyEmailCode(data: EmailVerifycationDto) {
    const { token, verifyCode, email, scene } = data
    const payload = this.jwtService.verify<EmailSendDto>(token, {
      secret: this.getEmailJwtSecret(verifyCode, scene),
    })
    if (payload.email !== email || payload.scene !== scene) {
      throw new ForbiddenException('邮箱验证码验证失败')
    }
  }

  getEmailJwtSecret(verifyCode: string, scene: EmailScene) {
    return this.secureConfig.jwt_access_secret + verifyCode + scene
  }

  private generateVerifyCode() {
    return Math.floor(Math.random() * 899999 + 100000).toString()
  }
}
